/*!
* rete v2.0.0-beta.4
* (c) 2023 Vitaliy Stoliarov
* Released under the MIT license.
* */
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

class Signal {
  constructor() {
    _defineProperty(this, "pipes", []);
  }
  addPipe(pipe) {
    this.pipes.push(pipe);
  }
  async emit(context) {
    let current = context;
    for (const pipe of this.pipes) {
      current = await pipe(current);
      if (typeof current === 'undefined') return;
    }
    return current;
  }
}
class Scope {
  // Parents['length'] extends 0 ? undefined : Scope<Parents[0], Tail<Parents>>

  constructor(name) {
    _defineProperty(this, "signal", new Signal());
    this.name = name;
  }
  addPipe(middleware) {
    this.signal.addPipe(middleware);
  }
  use(scope) {
    scope.setParent(this);
    this.addPipe(context => {
      return scope.signal.emit(context);
    });
  }
  setParent(scope) {
    this.parent = scope;
  }
  emit(context) {
    return this.signal.emit(context);
  }
  hasParent() {
    return Boolean(this.parent);
  }
  parentScope(type) {
    if (!this.parent) throw new Error('cannot find parent');
    if (type && this.parent instanceof type) return this.parent;
    if (type) throw new Error('actual parent is not instance of type');
    return this.parent;
  }
}

class NodeEditor extends Scope {
  constructor() {
    super('NodeEditor');
    _defineProperty(this, "nodes", []);
    _defineProperty(this, "connections", []);
  }
  getNode(id) {
    return this.nodes.find(node => node.id === id);
  }
  getNodes() {
    return this.nodes;
  }
  getConnections() {
    return this.connections;
  }
  getConnection(id) {
    return this.connections.find(connection => connection.id === id);
  }
  async addNode(data) {
    if (this.getNode(data.id)) throw new Error('node has already been added');
    if (!(await this.emit({
      type: 'nodecreate',
      data
    }))) return false;
    this.nodes.push(data);
    await this.emit({
      type: 'nodecreated',
      data
    });
    return true;
  }
  async addConnection(data) {
    if (this.getConnection(data.id)) throw new Error('connection has already been added');
    if (!(await this.emit({
      type: 'connectioncreate',
      data
    }))) return false;
    this.connections.push(data);
    await this.emit({
      type: 'connectioncreated',
      data
    });
    return true;
  }
  async removeNode(id) {
    const index = this.nodes.findIndex(n => n.id === id);
    const node = this.nodes[index];
    if (index < 0) throw new Error('cannot find node');
    if (!(await this.emit({
      type: 'noderemove',
      data: node
    }))) return false;
    this.nodes.splice(index, 1);
    await this.emit({
      type: 'noderemoved',
      data: node
    });
    return true;
  }
  async removeConnection(id) {
    const index = this.connections.findIndex(n => n.id === id);
    const connection = this.connections[index];
    if (index < 0) throw new Error('cannot find connection');
    if (!(await this.emit({
      type: 'connectionremove',
      data: connection
    }))) return false;
    this.connections.splice(index, 1);
    await this.emit({
      type: 'connectionremoved',
      data: connection
    });
    return true;
  }
  async clear() {
    if (!(await this.emit({
      type: 'clear'
    }))) {
      await this.emit({
        type: 'clearcancelled'
      });
      return false;
    }
    for (const connection of this.connections.slice()) await this.removeConnection(connection.id);
    for (const node of this.nodes.slice()) await this.removeNode(node.id);
    await this.emit({
      type: 'cleared'
    });
    return true;
  }
  async import(data) {
    if (!(await this.emit({
      type: 'import',
      data
    }))) return false;
    for (const node of data.nodes) await this.addNode(node);
    for (const connection of data.connections) await this.addConnection(connection);
    await this.emit({
      type: 'imported',
      data
    });
    return true;
  }
  async export() {
    const data = {
      nodes: [],
      connections: []
    };
    if (!(await this.emit({
      type: 'export',
      data
    }))) return false;
    data.nodes.push(...this.nodes);
    data.connections.push(...this.connections);
    await this.emit({
      type: 'exported',
      data
    });
    return data;
  }
}

/* global globalThis*/
const crypto = globalThis.crypto;
function getUID() {
  if ('randomBytes' in crypto) {
    return crypto.randomBytes(8).toString('hex');
  }
  const bytes = crypto.getRandomValues(new Uint8Array(8));
  const array = Array.from(bytes);
  const hexPairs = array.map(b => b.toString(16).padStart(2, '0'));
  return hexPairs.join('');
}

class Socket {
  constructor(name) {
    this.name = name;
  }
}
class Port {
  constructor(socket, label, multipleConnections) {
    this.socket = socket;
    this.label = label;
    this.multipleConnections = multipleConnections;
    this.id = getUID();
  }
}
class Input extends Port {
  constructor(...args) {
    super(...args);
    _defineProperty(this, "control", null);
    _defineProperty(this, "showControl", true);
  }
  addControl(control) {
    if (this.control) throw new Error('control already added for this input');
    this.control = control;
  }
  removeControl() {
    this.control = null;
  }
}
class Output extends Port {
  constructor(socket, label, multipleConnections) {
    super(socket, label, multipleConnections !== false);
  }
}
class Control {
  constructor() {
    this.id = getUID();
  }
}
class InputControl extends Control {
  constructor(type, options) {
    super();
    this.type = type;
    this.options = options;
    this.id = getUID();
    this.readonly = options === null || options === void 0 ? void 0 : options.readonly;
    if (typeof (options === null || options === void 0 ? void 0 : options.initial) !== 'undefined') this.value = options.initial;
  }
  setValue(value) {
    var _this$options;
    this.value = value;
    if ((_this$options = this.options) !== null && _this$options !== void 0 && _this$options.change) this.options.change(value);
  }
}
class Node {
  constructor(label) {
    _defineProperty(this, "inputs", {});
    _defineProperty(this, "outputs", {});
    _defineProperty(this, "controls", {});
    this.label = label;
    this.id = getUID();
  }
  hasInput(key) {
    return Object.prototype.hasOwnProperty.call(this.inputs, key);
  }
  addInput(key, input) {
    if (this.hasInput(key)) throw new Error(`input with key '${String(key)}' already added`);
    Object.defineProperty(this.inputs, key, {
      value: input,
      enumerable: true,
      configurable: true
    });
  }
  removeInput(key) {
    delete this.inputs[key];
  }
  hasOutput(key) {
    return Object.prototype.hasOwnProperty.call(this.outputs, key);
  }
  addOutput(key, output) {
    if (this.hasOutput(key)) throw new Error(`output with key '${String(key)}' already added`);
    Object.defineProperty(this.outputs, key, {
      value: output,
      enumerable: true,
      configurable: true
    });
  }
  removeOutput(key) {
    delete this.outputs[key];
  }
  hasControl(key) {
    return Object.prototype.hasOwnProperty.call(this.controls, key);
  }
  addControl(key, control) {
    if (this.hasControl(key)) throw new Error(`control with key '${String(key)}' already added`);
    Object.defineProperty(this.controls, key, {
      value: control,
      enumerable: true,
      configurable: true
    });
  }
  removeControl(key) {
    delete this.controls[key];
  }
}
class Connection {
  constructor(source, sourceOutput, target, targetInput) {
    this.sourceOutput = sourceOutput;
    this.targetInput = targetInput;
    if (!source.outputs[sourceOutput]) {
      throw new Error(`source node doesn't have output with a key ${String(sourceOutput)}`);
    }
    if (!target.inputs[targetInput]) {
      throw new Error(`target node doesn't have input with a key ${String(targetInput)}`);
    }
    this.id = getUID();
    this.source = source.id;
    this.target = target.id;
  }
}

var classic = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Socket: Socket,
  Port: Port,
  Input: Input,
  Output: Output,
  Control: Control,
  InputControl: InputControl,
  Node: Node,
  Connection: Connection
});

export { classic as ClassicPreset, NodeEditor, Scope, Signal, getUID };
//# sourceMappingURL=rete.esm.js.map
